/*----------------------------------------------------------------------------------
% Authors: V. Bianchi, M. Zampetti
%          A. Canessa, M. Chessa, F. Solari
% PSPC-lab - Biophysical and Electronic Engineering Department - University of Genoa
% September 2010
------------------------------------------------------------------------------------*/

#include "PopCod_Disp2D.h"
#include <cstdio>
#include <cstdlib>
#include <cmath>
using namespace std;



PopCod_Disp2D::PopCod_Disp2D(string filter_name, int width, int height, int n_scales, int n_phases, float dmax, float energy_th, int ori_thr){

	m=height;  n=width;  n_liv=n_scales;  n_phase=n_phases;  nc_min=ori_thr;
	n_frame=1; n_vel=0; n_orient=8; dx_sx=1;
	energy_thr=energy_th;
	//-------------------------------------------------------------------------------------------------------
	allocate_buffer(n_liv, n_frame, dx_sx, n_orient, n_phase, m, n, &buff);
	allocate_buffer(n_liv, n_frame, dx_sx, n_orient, n_phase, m, n, &copy_buff);
	allocate_for_energy(&energy, n_liv, n_orient, n_phase, n_vel, m, n);
	allocate_for_decomp_energy(&dec_energy, n_liv, n_orient, n_phase, m, n);
	allocate_for_population(&population, n_liv, n_orient, m, n);
	allocate_for_population(&mask, n_liv, n_orient, m, n);
	allocate_for_decomposition_of_population(&dec_population, n_liv, n_orient, m, n);
	allocate_for_fulldisparity(&full_disparity, n_liv, m, n);
	allocate_for_fulldisparity(&copy_full, (n_liv-1), m, n);
	allocate_for_pyramid(&pyramsx, n_liv, m, n);
	allocate_for_pyramid(&pyramdx, n_liv, m, n);  

	//--------Loading of the Gabor filters------------------------------------------------------------------------
	load_gabor_filters(filter_name);

	//--------Computation of the disparity tuning values for the population cells and of the LP filter for pyramid---------------
	disparity.m=1; disparity.n=n_phase;
	disparity.ptr=ippiMalloc_32f_C1(disparity.n, disparity.m, &(disparity.stp));
	for(int i=0;i<n_phase;i++)
		disparity.ptr[i]=-dmax + i*(2*dmax/(n_phase-1));

	filpyr.m=1; filpyr.n=5;
	filpyr.ptr=ippiMalloc_32f_C1(filpyr.n, filpyr.m, &(filpyr.stp));
	filpyr.ptr[0]=0.0625f; filpyr.ptr[1]=0.2500f; filpyr.ptr[2]=0.3750f; filpyr.ptr[3]=0.2500f; filpyr.ptr[4]=0.0625f;

	PI_greco=4.0f*atanf(1.0f);
	mask2.ptr=0; copymask2.ptr=0;

	//-----info---------------------------------------------------------
	printf("\nImages: width=%d, height=%d; %d scales\n",width,height,n_liv);
	printf("Filters: %dx%d;  fo=%f; taps=%d\n",n_fil_spaz,taps,fo,taps);
	printf("Disp: ");
	for(int i=0;i<n_phase;i++)
		printf("%.2f  ",disparity.ptr[i]);
	printf("\n");
	printf("FilPyr: ");
	for(int i=0;i<5;i++)
		printf("%.4f  ",filpyr.ptr[i]);
	printf("\n");
}

PopCod_Disp2D::~PopCod_Disp2D(){
	//------------------------------------------------------------------------------
	deallocate_buffer(n_liv, n_frame, dx_sx, n_orient, n_phase, &buff);
	deallocate_buffer(n_liv, n_frame, dx_sx, n_orient, n_phase, &copy_buff);  
	deallocate_for_energy(&energy, n_liv, n_orient, n_vel, n_phase);
	deallocate_for_decomp_energy(&dec_energy, n_liv, n_orient, n_phase);
	deallocate_for_population(&population, n_liv, n_orient);
	deallocate_for_population(&mask, n_liv, n_orient);
	deallocate_for_decomposition_of_population(&dec_population, n_liv, n_orient);     
	deallocate_for_fulldisparity(&full_disparity, n_liv);
	deallocate_for_fulldisparity(&copy_full, (n_liv-1));
	deallocate_for_pyramid(&pyramsx, n_liv);
	deallocate_for_pyramid(&pyramdx, n_liv);
	//-----------------------------------------------------------------------------------
	for (int i=0;i<n_fil_spaz;i++)
		ippiFree(fil_spaz[i].ptr);
	free(fil_spaz);
	//
	ippiFree(disparity.ptr);
	ippiFree(filpyr.ptr);
}


void PopCod_Disp2D::img_ipp_alloc_frombin(const Ipp32f *imgbin, MatrixIpp *imgipp){
	imgipp->m=m; 	imgipp->n=n;
	imgipp->ptr = ippiMalloc_32f_C1(imgipp->n, imgipp->m, &(imgipp->stp));
	IppiSize size;
	size.height=imgipp->m; size.width=imgipp->n;
	ippiCopy_32f_C1R((const Ipp32f*)imgbin,sizeof(Ipp32f)*imgipp->n,imgipp->ptr,imgipp->stp,size);
}
void PopCod_Disp2D::img_ipp_dealloc( MatrixIpp *img){
	ippiFree(img->ptr);
}
void PopCod_Disp2D::img_ipp_2bin(const MatrixIpp *img,Ipp32f *imgdst){
	IppiSize size;
	size.height=img->m; size.width=img->n;
	ippiCopy_32f_C1R(img->ptr,img->stp,imgdst,sizeof(Ipp32f)*img->n,size);
}
void PopCod_Disp2D::load_gabor_filters(std::string filter_filename){
	FILE *fp;
	fp=fopen(filter_filename.c_str(),"rb");
	if (fp==NULL){
		printf("Error opening file: %s\n",filter_filename.c_str());
		exit(1);
	}
	float  tapss, B;
	fread(&fo,sizeof(float),1,fp);
	fread(&tapss,sizeof(float),1,fp);
	fread(&B,sizeof(float),1,fp);
	taps=(int)tapss;
	float rr, cc;
	fread(&rr,sizeof(float),1,fp);fread(&cc,sizeof(float),1,fp);
	n_fil_spaz=(int)rr;
	fil_spaz =(struct MatrixIpp *)malloc(n_fil_spaz * sizeof(struct MatrixIpp)); 
	float *data=(float*) malloc(sizeof(float)*n_fil_spaz*taps);
	fread(data,sizeof(float),n_fil_spaz*taps,fp);
	fclose(fp);
	for (int i=0;i<n_fil_spaz;i++){
		fil_spaz[i].m=1;
		fil_spaz[i].n=taps;
		fil_spaz[i].ptr = ippiMalloc_32f_C1(fil_spaz[i].n, fil_spaz[i].m, &(fil_spaz[i].stp));
		for(int j=0;j<fil_spaz[i].n;j++)
			fil_spaz[i].ptr[j]=data[i*taps+j];
	}
	free(data);
}
void PopCod_Disp2D::Compute_maps(const Ipp32f *left_img, const Ipp32f *right_img, Ipp32f *d_x, Ipp32f *d_y){
	const char *c="samerepeat";
	float ph_extr=fabs(disparity.ptr[0]);
	float ph_step=fabs(disparity.ptr[0]-disparity.ptr[1]);
	//----------input: left & right images-----------------------------------------------------------------
	MatrixIpp L, R;
	img_ipp_alloc_frombin(left_img,&L);
	img_ipp_alloc_frombin(right_img,&R);

	//----------computation of the disparity maps--------------------------------------------------------------------
	pyramid_no_alloc(L, filpyr, filpyr, &pyramsx, n_liv, c);
	pyramid_no_alloc(R, filpyr, filpyr, &pyramdx, n_liv, c); 

	filt_spatial_gabor_no_alloc(pyramsx[0], fil_spaz, &buff[0][0][SX], c);           

	filt_spatial_gabor_no_alloc(pyramdx[0], fil_spaz, &buff[0][0][DX], c);

	shift_in_phase(ph_extr, ph_step, buff[0][0][0], &buff[0][0][0], n_orient);	

	energy_no_alloc(buff[0][0], &energy[0], &dec_energy[0], n_orient, n_phase,energy_thr);

	population_no_alloc(energy[0], disparity, &population[0], &mask[0], n_orient, n_phase);

	decomposition_of_population_no_alloc(population[0], &dec_population[0], n_orient);

	fulldisparity_no_alloc(dec_population[0], &full_disparity[0], mask[0],&mask2, n_orient, nc_min); 

	for(int i = 1 ; i < n_liv ; i++)        
	{		
		//printf("\n i=%d\n",i);
		expand_mulX2_fulldisparity_no_alloc(full_disparity[i - 1], &full_disparity[i]);

		expand_mask(mask2, &copymask2, full_disparity[i][0].m, full_disparity[i][0].n);

		filt_spatial_gabor_no_alloc(pyramsx[i], fil_spaz, &buff[i][0][1], c);

		filt_spatial_gabor_no_alloc(pyramdx[i], fil_spaz, &buff[i][0][0], c);

		shift_in_phase(ph_extr, ph_step, buff[i][0][0], &buff[i][0][0], n_orient);

		warping_frame_filt_gabor(buff[i][0], &(copy_buff[i][0]), full_disparity[i], n_frame, n_orient, n_phase);

		energy_no_alloc(copy_buff[i][0], &energy[i],&dec_energy[i], n_orient, n_phase,energy_thr);

		population_no_alloc(energy[i], disparity, &population[i], &mask[i], n_orient, n_phase);

		decomposition_of_population_no_alloc(population[i], &dec_population[i], n_orient);

		fulldisparity_no_alloc(dec_population[i], &copy_full[i-1],mask[i],&mask2, n_orient, nc_min);

		merge(copy_full[i-1], &full_disparity[i]);

		int temp=mask2.stp/sizeof(Ipp32f);
		for(int j=0; j<mask2.m;j++)
			for(int k=0;k<mask2.n;k++)
			{
				if(mask2.ptr[(j*temp) + k]==0 && copymask2.ptr[(j*temp) + k]==0)
					mask2.ptr[(j*temp) + k]=0;
				//else  //mdf
					//mask2.ptr[(j*temp) + k]=1;
			}

	}

	cut_invalid_at_end_flow(mask2, &full_disparity[n_liv-1],nc_min);

	ippiFree(mask2.ptr); mask2.ptr=0;
	ippiFree(copymask2.ptr); copymask2.ptr=0;

	//----------output: disparity maps----------------------------------------------------------------------
	img_ipp_2bin(&full_disparity[n_liv-1][0],d_x);
	img_ipp_2bin(&full_disparity[n_liv-1][1],d_y);

	img_ipp_dealloc(&L);
	img_ipp_dealloc(&R);

}
//---------------------------------------------------------------------------------------------------------------------
//*********************************************************************************************************************
//---------------------------------------------------------------------------------------------------------------------


void PopCod_Disp2D::allocate_buffer(int n_liv_pir, int n_frame, int sx_dx, int n_orient, int n_phase, int m_imm, int n_imm, struct MatrixIpp *******buffer)
{
	struct MatrixIpp ******temp_buffer;
	int i, j, k, h,l;
	IppiSize size;
	IppStatus err;


	temp_buffer = (struct MatrixIpp ******)malloc(n_liv_pir * sizeof(struct MatrixIpp *****)); 

	for(i = 0 ; i < n_liv_pir ; i++)
		temp_buffer[i] = (struct MatrixIpp *****)malloc(n_frame * sizeof(struct MatrixIpp ****)); 

	for(i = 0 ; i < n_liv_pir ; i++)
		for(j = 0 ; j < n_frame ; j++)
			temp_buffer[i][j] = (struct MatrixIpp ****)malloc(2 * sizeof(struct MatrixIpp ***)); //abbiamo portato fuori dall'else il for e abbiamo allocato per  2 per avere lo spazio sia per l'imm dx sia per l'imm dx


	if(sx_dx != 0)
	{
		for(i = 0 ; i < n_liv_pir ; i++)
			for(j = 0 ; j < n_frame ; j++)
			{
				temp_buffer[i][j][0] = (struct MatrixIpp ***)malloc(2 * sizeof(struct MatrixIpp **)); 

			}

			for(i = 0 ; i < n_liv_pir ; i++)
				for(j = 0 ; j < n_frame ; j++)
					for(k = 0 ; k < 2 ; k++)
					{
						temp_buffer[i][j][0][k] = (struct MatrixIpp **)malloc(n_orient * sizeof(struct MatrixIpp*)); 

					}

					for(i = 0 ; i < n_liv_pir ; i++)
						for(j = 0 ; j < n_frame ; j++)
							for(k = 0 ; k < 2 ; k++)
								for(h = 0 ; h < n_orient ; h++)
								{
									temp_buffer[i][j][0][k][h] = (struct MatrixIpp *)malloc(n_phase * sizeof(struct MatrixIpp)); 

								} 

								for(i = 0 ; i < n_liv_pir ; i++)
									for(j = 0 ; j < n_frame ; j++)
										for(k = 0 ; k < 2 ; k++)
											for(h = 0 ; h<n_orient ; h++)
												for(l = 0 ; l < n_phase ; l++)
												{
													temp_buffer[i][j][0][k][h][l].m = m_imm / (int)pow(2.0f,(n_liv_pir - i - 1));
													temp_buffer[i][j][0][k][h][l].n = n_imm / (int)pow(2.0f,(n_liv_pir - i - 1));
													temp_buffer[i][j][0][k][h][l].ptr = ippiMalloc_32f_C1(temp_buffer[i][j][0][k][h][l].n, temp_buffer[i][j][0][k][h][l].m, &(temp_buffer[i][j][0][k][h][l].stp));

													size.width =  temp_buffer[i][j][0][k][h][l].n;
													size.height = temp_buffer[i][j][0][k][h][l].m;
													err = ippiSet_32f_C1R(0, temp_buffer[i][j][0][k][h][l].ptr, temp_buffer[i][j][0][k][h][l].stp, size);
													if (err != ippStsOk)
														printf("ERRORE  allocate_buffer_persistent a sinistra");
												}
	}

	//l'immagine sinistra non viene sfasata ==> k=0
	for(i = 0 ; i < n_liv_pir ; i++)
		for(j = 0 ; j < n_frame ; j++)              //allochiamo per parte re e im
		{
			temp_buffer[i][j][1] = (struct MatrixIpp ***)malloc(2 * sizeof(struct MatrixIpp **)); 

		}

		for(i = 0 ; i < n_liv_pir ; i++)
			for(j = 0 ; j < n_frame ; j++)
				for(k = 0 ; k < 2 ; k++)
				{
					temp_buffer[i][j][1][k] = (struct MatrixIpp **)malloc(n_orient * sizeof(struct MatrixIpp*)); 

				}

				for(i = 0 ; i < n_liv_pir ; i++)
					for(j = 0 ; j < n_frame ; j++)
						for(k = 0 ; k < 2 ; k++)
							for(h = 0 ; h < n_orient ; h++)
							{
								temp_buffer[i][j][1][k][h] = (struct MatrixIpp *)malloc(1 * sizeof(struct MatrixIpp)); 

								temp_buffer[i][j][1][k][h][0].m = m_imm / (int)pow(2.0f,(n_liv_pir - i - 1));
								temp_buffer[i][j][1][k][h][0].n = n_imm / (int)pow(2.0f,(n_liv_pir - i - 1));
								temp_buffer[i][j][1][k][h][0].ptr = ippiMalloc_32f_C1(temp_buffer[i][j][1][k][h][0].n, temp_buffer[i][j][1][k][h][0].m, &(temp_buffer[i][j][1][k][h][0].stp));

								size.width =  temp_buffer[i][j][1][k][h][0].n;
								size.height = temp_buffer[i][j][1][k][h][0].m;
								err = ippiSet_32f_C1R(0, temp_buffer[i][j][1][k][h][0].ptr, temp_buffer[i][j][1][k][h][0].stp, size);
								if (err != ippStsOk)
									printf("ERRORE  allocate_buffer_persistent a destra");
							}
							*buffer=temp_buffer;

}

void PopCod_Disp2D::deallocate_buffer(int n_liv_pir, int n_frame, int sx_dx, int n_orient, int n_phase, struct MatrixIpp *******buffer)
{
	struct MatrixIpp ******temp_buffer;
	int i, j, k, h,l;

	temp_buffer = *buffer;

	for(i = 0 ; i < n_liv_pir ; i++)
		for(j = 0 ; j < n_frame ; j++)
		{
			for(k = 0 ; k < 2 ; k++)
			{
				for(h = 0 ; h < n_orient ; h++){
					ippiFree(temp_buffer[i][j][1][k][h][0].ptr);
					free(temp_buffer[i][j][1][k][h]);//<--
				}
				free(temp_buffer[i][j][1][k]);
			}
			free(temp_buffer[i][j][1]);
		}

		if(sx_dx != 0)
		{
			for(i = 0 ; i < n_liv_pir ; i++)
				for(j = 0 ; j < n_frame ; j++)
				{
					for(k = 0 ; k <2  ; k++)
					{
						for(l = 0 ; l < n_orient ; l++)                   
						{
							for(h = 0 ; h <n_phase  ; h++)
								ippiFree(temp_buffer[i][j][0][k][l][h].ptr);
							free(temp_buffer[i][j][0][k][l]);
						}
						free(temp_buffer[i][j][0][k]);
					}
					free(temp_buffer[i][j][0]);
				}
		}

		for(i = 0 ; i < n_liv_pir ; i++)
		{
			for(j = 0 ; j < n_frame ; j++)
			{
				free(temp_buffer[i][j]);
			}
			free(temp_buffer[i]);//<--
		}

		free(temp_buffer);
}
void PopCod_Disp2D::allocate_for_energy(struct MatrixIpp ****ris, int n_liv, int n_orient, int n_phase, int n_vel, int m, int n)
{
	struct MatrixIpp ***temp_ris;

	int i, j, w;

	temp_ris = (struct MatrixIpp ***)malloc(n_liv * sizeof(struct MatrixIpp **)); 

	for(i = 0 ; i < n_liv ; i++)
	{
		temp_ris[i] = (struct MatrixIpp **)malloc(n_orient * sizeof(struct MatrixIpp *)); 

	}

	if(n_phase != 0)
	{
		for(j = 0 ; j < n_liv ; j++)
			for(i = 0 ; i < n_orient ; i++)
			{
				temp_ris[j][i] = (struct MatrixIpp *)malloc(n_phase * sizeof(struct MatrixIpp)); 

			}

			for(w = 0 ; w < n_liv ; w++)
				for(i = 0 ; i < n_orient ; i++)
					for(j = 0 ; j < n_phase ; j++)
					{
						temp_ris[w][i][j].m = m / (int)pow(2.0f,(n_liv - w - 1));
						temp_ris[w][i][j].n = n / (int)pow(2.0f,(n_liv - w - 1));
						temp_ris[w][i][j].ptr = ippiMalloc_32f_C1(temp_ris[w][i][j].n, temp_ris[w][i][j].m, &(temp_ris[w][i][j].stp));
					}
	}
	else
	{
		for(j = 0 ; j < n_liv ; j++)
			for(i = 0 ; i < n_orient ; i++)
			{
				temp_ris[j][i] = (struct MatrixIpp *)malloc(n_vel * sizeof(struct MatrixIpp)); 
			}

			for(w = 0 ; w < n_liv ; w++)
				for(i = 0 ; i < n_orient ; i++)
					for(j = 0 ; j < n_vel ; j++)
					{
						temp_ris[w][i][j].m = m / (int)pow(2.0f,(n_liv - w - 1));
						temp_ris[w][i][j].n = n / (int)pow(2.0f,(n_liv - w - 1));
						temp_ris[w][i][j].ptr = ippiMalloc_32f_C1(temp_ris[w][i][j].n, temp_ris[w][i][j].m, &(temp_ris[w][i][j].stp));
					}
	}
	*ris = temp_ris;
}
void PopCod_Disp2D::deallocate_for_energy(struct MatrixIpp ****ris, int n_liv, int n_orient, int n_vel, int n_phase)
{
	struct MatrixIpp ***temp_ris;
	int i, j, w;

	temp_ris = *ris;

	if(n_phase != 0)
	{
		for(w = 0; w < n_liv ; w++)
		{
			for(i = 0 ; i < n_orient ; i++)
			{
				for(j = 0 ; j < n_phase ; j++)
					ippiFree(temp_ris[w][i][j].ptr);
				free(temp_ris[w][i]);
			}
			free(temp_ris[w]);
		}
	}
	else
	{
		for(w = 0; w < n_liv ; w++)
		{
			for(i = 0 ; i < n_orient ; i++)
			{
				for(j = 0 ; j < n_vel ; j++)
					ippiFree(temp_ris[w][i][j].ptr);
				free(temp_ris[w][i]);
			}
			free(temp_ris[w]);
		}
	}
	free(temp_ris);
}
void PopCod_Disp2D::allocate_for_decomp_energy(struct MatrixIpp *****ris, int n_liv, int n_orient, int n_phase, int m, int n)
{
	struct MatrixIpp ****temp_ris;

	int i, j, h;

	temp_ris = (struct MatrixIpp ****)malloc(n_liv * sizeof(struct MatrixIpp ***)); 

	for(i = 0 ; i < n_liv ; i++)
	{
		temp_ris[i] = (struct MatrixIpp ***)malloc(n_orient * sizeof(struct MatrixIpp **)); 
	}

	for(j = 0 ; j < n_liv ; j++)
		for(i = 0 ; i < n_orient ; i++)
		{
			temp_ris[j][i] = (struct MatrixIpp **)malloc(2 * sizeof(struct MatrixIpp *)); 
		}

		for(j = 0 ; j < n_liv ; j++)
			for(i = 0 ; i < n_orient ; i++)
			{
				temp_ris[j][i][1] = (struct MatrixIpp *)malloc(1 *sizeof(struct MatrixIpp)); 

				temp_ris[j][i][1][0].m = m / (int)pow(2.0f,(n_liv - j - 1));
				temp_ris[j][i][1][0].n = n / (int)pow(2.0f,(n_liv - j - 1));
				temp_ris[j][i][1][0].ptr = ippiMalloc_32f_C1(temp_ris[j][i][1][0].n, temp_ris[j][i][1][0].m, &(temp_ris[j][i][1][0].stp));
			}
			for(j = 0 ; j < n_liv ; j++)
				for(i = 0 ; i < n_orient ; i++)
				{
					temp_ris[j][i][0] = (struct MatrixIpp *)malloc(n_phase * sizeof(struct MatrixIpp)); 
				}
				for(j = 0 ; j < n_liv ; j++)
					for(i = 0 ; i < n_orient ; i++)
						for(h = 0 ; h < n_phase ; h++)
						{
							temp_ris[j][i][0][h].m = m / (int)pow(2.0f,(n_liv - j - 1));
							temp_ris[j][i][0][h].n = n / (int)pow(2.0f,(n_liv - j - 1));
							temp_ris[j][i][0][h].ptr = ippiMalloc_32f_C1(temp_ris[j][i][0][h].n, temp_ris[j][i][0][h].m, &(temp_ris[j][i][0][h].stp));
						}
						*ris = temp_ris;
}
void PopCod_Disp2D::deallocate_for_decomp_energy(struct MatrixIpp *****ris, int n_liv, int n_orient, int n_phase)
{
	int i, j, h;
	struct MatrixIpp ****temp_ris;

	temp_ris = *ris;

	for(j = 0 ; j < n_liv ; j++)
	{
		for(i = 0 ; i < n_orient ; i++)
		{
			ippiFree(temp_ris[j][i][1][0].ptr);
			free(temp_ris[j][i][1]);
			for(h = 0 ; h < n_phase ; h++)
				ippiFree(temp_ris[j][i][0][h].ptr);
			free(temp_ris[j][i][0]);
			free(temp_ris[j][i]);
		}
		free(temp_ris[j]);
	}
	free(temp_ris);
}
//
void PopCod_Disp2D::allocate_for_population(struct MatrixIpp ***ris, int n_liv, int n_orient, int m, int n)
{
	struct MatrixIpp **temp_ris;
	int i, j;

	temp_ris = (struct MatrixIpp **)malloc(n_liv * sizeof(struct MatrixIpp *)); 

	for(i = 0 ; i < n_liv ; i++)
	{
		temp_ris[i] = (struct MatrixIpp *)malloc(n_orient * sizeof(struct MatrixIpp)); 
	}

	for(i = 0 ; i < n_liv ; i++)
		for(j = 0 ; j < n_orient ; j++)
		{
			temp_ris[i][j].m = m / (int)pow(2.0f,(n_liv - i - 1));
			temp_ris[i][j].n = n / (int)pow(2.0f,(n_liv - i - 1));
			temp_ris[i][j].ptr = ippiMalloc_32f_C1(temp_ris[i][j].n, temp_ris[i][j].m, &(temp_ris[i][j].stp));
		}

		*ris = temp_ris;
}
void PopCod_Disp2D::deallocate_for_population(struct MatrixIpp ***ris, int n_liv, int n_orient)
{
	struct MatrixIpp **temp_ris;
	int i, j;
	temp_ris = *ris;

	for(i = 0 ; i < n_liv ; i++)
	{
		for(j = 0 ; j < n_orient ; j++)
			ippiFree(temp_ris[i][j].ptr);
		free(temp_ris[i]);
	}
	free(temp_ris);
}
void PopCod_Disp2D::allocate_for_decomposition_of_population(struct MatrixIpp ****ris, int n_liv, int n_orient, int m, int n)
{
	struct MatrixIpp ***temp_ris;
	int i, j, k;

	temp_ris = (struct MatrixIpp ***)malloc(n_liv * sizeof(struct MatrixIpp **)); 

	for(i = 0 ; i < n_liv ; i++)
	{
		temp_ris[i] = (struct MatrixIpp **)malloc(n_orient * sizeof(struct MatrixIpp *)); 
	}

	for(i = 0 ; i < n_liv ; i++)
		for(j = 0 ; j < n_orient ; j++)
		{
			temp_ris[i][j] = (struct MatrixIpp *)malloc(2 * sizeof(struct MatrixIpp)); 
		}

		for(i = 0 ; i < n_liv ; i++)
			for(j = 0 ; j < n_orient ; j++)
				for(k = 0 ; k < 2 ; k++)
				{
					temp_ris[i][j][k].m = m / (int)pow(2.0f,(n_liv - i - 1));
					temp_ris[i][j][k].n = n / (int)pow(2.0f,(n_liv - i - 1));
					temp_ris[i][j][k].ptr = ippiMalloc_32f_C1(temp_ris[i][j][k].n, temp_ris[i][j][k].m, &(temp_ris[i][j][k].stp));
				}

				*ris = temp_ris;
}
void PopCod_Disp2D::deallocate_for_decomposition_of_population(struct MatrixIpp ****ris, int n_liv, int n_orient)
{
	struct MatrixIpp ***temp_ris;
	int i, j, k;

	temp_ris = *ris;

	for(i = 0 ; i < n_liv ; i++)
	{
		for(j = 0 ; j < n_orient ; j++)
		{
			for(k = 0 ; k < 2 ; k++)
				ippiFree(temp_ris[i][j][k].ptr);
			free(temp_ris[i][j]);
		}
		free(temp_ris[i]);
	}
	free(temp_ris);
}
//
void PopCod_Disp2D::allocate_for_fulldisparity(struct MatrixIpp ***ris, int n_liv, int m, int n)
{
	struct MatrixIpp **temp_ris;
	int i, j;

	temp_ris = (struct MatrixIpp **)malloc(n_liv * sizeof(struct MatrixIpp *)); 

	for(i = 0 ; i < n_liv ; i++)
	{
		temp_ris[i] = (struct MatrixIpp *)malloc(2 * sizeof(struct MatrixIpp)); 
	}

	for(i = 0 ; i < n_liv ; i++)
		for(j = 0 ; j < 2 ; j++)
		{
			temp_ris[i][j].m = m / (int)pow(2.0f,(n_liv - i - 1));
			temp_ris[i][j].n = n / (int)pow(2.0f,(n_liv - i - 1));
			temp_ris[i][j].ptr = ippiMalloc_32f_C1(temp_ris[i][j].n, temp_ris[i][j].m, &(temp_ris[i][j].stp));
		}
		*ris = temp_ris;

}

void PopCod_Disp2D::deallocate_for_fulldisparity(struct MatrixIpp ***ris, int n_liv)
{
	struct MatrixIpp **temp_ris;
	int i, j;

	temp_ris = *ris;

	for(i = 0 ; i < n_liv ; i++)
	{
		for(j = 0 ; j < 2 ; j++)
			ippiFree(temp_ris[i][j].ptr);
		free(temp_ris[i]);
	}
	free(temp_ris);
}
void PopCod_Disp2D::allocate_for_pyramid(struct MatrixIpp **ris, int n_liv, int m, int n)
{
	struct MatrixIpp *temp_ris;
	int i;

	temp_ris = (struct MatrixIpp *)malloc(n_liv * sizeof(struct MatrixIpp)); 


	for(i = 0 ; i < n_liv ; i++)
	{
		temp_ris[i].m = m / (int)pow(2.0f,(n_liv - i - 1));
		temp_ris[i].n = n / (int)pow(2.0f,(n_liv - i - 1));
		temp_ris[i].ptr = ippiMalloc_32f_C1(temp_ris[i].n, temp_ris[i].m, &(temp_ris[i].stp));

	}
	*ris = temp_ris;
}

void PopCod_Disp2D::deallocate_for_pyramid(struct MatrixIpp **pir, int n_liv)
{
	struct MatrixIpp *temp_pir;
	int i;

	temp_pir = *pir;

	for(i = 0 ; i < n_liv ; i++)
		ippiFree(temp_pir[i].ptr);
	free(temp_pir);
}
//------------------------------------------------------------------------------------------------------------------------------------
//---------pyramid--------------------------------------------------------------------------------------------------------------------
void PopCod_Disp2D::pyramid_no_alloc(struct MatrixIpp image, struct MatrixIpp fil_a, struct MatrixIpp fil_b, struct MatrixIpp **pir, int n_level, const char *conv_type)
{
	struct MatrixIpp *temp_pir, temp_a, temp_b;
	int i;
	int off_m, off_n;

	temp_pir = *pir;

	copy_matrixipp_no_alloc(image, &temp_pir[n_level - 1]);

	for(i = n_level - 1 ; i > 0 ; i--)
	{      
		off_m = fil_a.n;
		off_n = fil_a.n;

		padding_alloc(temp_pir[i], &temp_a, off_m, off_n, conv_type);

		convolution2D_valid_allocr(temp_a, fil_a, &temp_b);
		ippiFree(temp_a.ptr);

		convolution2D_valid_allocc(temp_b, fil_b, &temp_a);
		ippiFree(temp_b.ptr);

		dimez_matrixipp_no_alloc(temp_a, &temp_pir[i - 1]);
		ippiFree(temp_a.ptr);
	}

	*pir = temp_pir;
}
void PopCod_Disp2D::copy_matrixipp_no_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest)
{
	int temp, sstr0, sstr1, sstr2, rstr0, rstr1, rstr2;
	IppStatus err;

	temp = sizeof(Ipp32f);
	sstr0 = sorg.m  * sorg.stp;
	sstr1 = sorg.stp;
	sstr2 = temp;
	rstr0 = dest->m * dest->stp;
	rstr1 = dest->stp;
	rstr2 = temp;

	temp = 1;
	err = ippmCopy_ma_32f_SS(sorg.ptr, sstr0, sstr1, sstr2, dest->ptr, rstr0, rstr1, rstr2, dest->n, dest->m, temp);

	if (err != ippStsOk) 
		printf("ERRORE  in  copy_matrixipp_no_alloc");
}
void PopCod_Disp2D::padding_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest, int m_fil, int n_fil, const char *type)
{   
	if((strcmp(type,"samezero"))==0)
	{
		zero_padding_alloc(sorg,dest,m_fil,n_fil);
	}
	else if((strcmp(type,"samerepeat"))==0)
	{
		repeat_padding_alloc(sorg,dest,m_fil,n_fil);
	}    
	else
		printf("padding_alloc(): Type is required.");
}
void PopCod_Disp2D::zero_padding_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest, int m_fil, int n_fil)
{
	int temp, off_m, off_n, frame;
	int sstr0, sstr1, sstr2, rstr0, rstr1, rstr2;
	Ipp32f *ptr_temp;
	IppiSize app;
	IppStatus err, err1;


	dest->m = sorg.m + m_fil - 1;
	dest->n = sorg.n + n_fil - 1;
	dest->ptr = ippiMalloc_32f_C1(dest->n, dest->m, &(dest->stp));
	app.width = dest->n;
	app.height = dest->m;
	err1 = ippiSet_32f_C1R(0, dest->ptr, dest->stp, app);


	off_m = m_fil / 2;  
	off_n = n_fil / 2;

	temp = sizeof(Ipp32f);
	sstr0 = sorg.m * sorg.stp;
	sstr1 = sorg.stp;
	sstr2 = temp;
	rstr0 = dest->m * dest->stp;
	rstr1 = dest->stp;
	rstr2 = temp;

	frame = 1;
	ptr_temp = dest->ptr + off_n + (off_m * dest->stp / temp);

	err = ippmCopy_ma_32f_SS(sorg.ptr, sstr0, sstr1, sstr2, ptr_temp, rstr0, rstr1, rstr2, sorg.n, sorg.m, frame);

	if ((err != ippStsOk) || (err1 != ippStsNoErr))
		printf("ERRORE  zero_padding");
}

void PopCod_Disp2D::repeat_padding_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest, int m_fil, int n_fil)
{
	int temp, off_m, off_n, frame, i;
	int sstr0, sstr1, sstr2, rstr0, rstr1, rstr2;
	Ipp32f *ptr_temp_srg, *ptr_temp_dst, value;
	IppiSize app;
	IppStatus err0, err1, err2, err3, err4, err5, err6, err7, err8;

	dest->m = sorg.m + m_fil - 1;
	dest->n = sorg.n + n_fil - 1;
	dest->ptr = ippiMalloc_32f_C1(dest->n, dest->m, &(dest->stp));

	off_m = m_fil / 2;  
	off_n = n_fil / 2;

	temp = sizeof(Ipp32f);
	sstr0 = sorg.m * sorg.stp;
	sstr1 = sorg.stp;
	sstr2 = temp;
	rstr0 = dest->m * dest->stp;
	rstr1 = dest->stp;
	rstr2 = temp;

	frame = 1;
	ptr_temp_dst = dest->ptr + off_n + (off_m * dest->stp / temp);

	err0 = ippmCopy_ma_32f_SS(sorg.ptr, sstr0, sstr1, sstr2, ptr_temp_dst, rstr0, rstr1, rstr2, sorg.n, sorg.m, frame);

	err0 = ippStsOk;
	err1 = ippStsOk;
	err2 = ippStsOk;
	err3 = ippStsOk;
	err4 = ippStsOk;    

	// ALTO
	sstr0 = sorg.stp;
	sstr2 = temp;
	rstr0 = dest->stp;
	rstr2 = temp;
	ptr_temp_srg = sorg.ptr;
	ptr_temp_dst = dest->ptr + off_n;

	for(i = 0 ; i < off_m ; i++)
	{
		err1 = ippmCopy_va_32f_SS(ptr_temp_srg, sstr0, sstr2, ptr_temp_dst, rstr0, rstr2, sorg.n, frame); 
		ptr_temp_dst += dest->stp / temp;
	}

	//SINISTRA
	sstr0 = sorg.stp * sorg.m;
	sstr2 = sorg.stp;
	rstr0 = dest->stp * sorg.m;
	rstr2 = dest->stp;
	ptr_temp_srg = sorg.ptr;
	ptr_temp_dst = dest->ptr + (off_m * dest->stp / temp);

	for(i = 0 ; i < off_n ; i++)
	{
		err2 = ippmCopy_va_32f_SS(ptr_temp_srg, sstr0, sstr2, ptr_temp_dst, rstr0, rstr2, sorg.m, frame); 
		ptr_temp_dst += 1;
	}

	//DESTRA
	sstr0 = sorg.stp * sorg.m;
	sstr2 = sorg.stp;
	rstr0 = dest->stp * sorg.m;
	rstr2 = dest->stp;
	ptr_temp_srg = sorg.ptr + sorg.n - 1;
	ptr_temp_dst = dest->ptr + (off_m * dest->stp / temp) + off_n + sorg.n;

	for(i = 0 ; i < off_n ; i++)
	{
		err3 = ippmCopy_va_32f_SS(ptr_temp_srg, sstr0, sstr2, ptr_temp_dst, rstr0, rstr2, sorg.m, frame); 
		ptr_temp_dst += 1;
	}

	//BASSO
	sstr0 = sorg.stp;
	sstr2 = temp;
	rstr0 = dest->stp;
	rstr2 = temp;
	ptr_temp_srg = sorg.ptr + ((sorg.m - 1) * (sorg.stp / temp));
	ptr_temp_dst = dest->ptr + off_n + (off_m * dest->stp / temp) + ((sorg.m) * (dest->stp / temp));

	for(i = 0 ; i < off_m ; i++)
	{
		err4 = ippmCopy_va_32f_SS(ptr_temp_srg, sstr0, sstr2, ptr_temp_dst, rstr0, rstr2, sorg.n, frame); 
		ptr_temp_dst += dest->stp / temp;
	}

	//quadratini negli angoli
	err5 = ippStsNoErr;
	err6 = ippStsNoErr;
	err7 = ippStsNoErr;
	err8 = ippStsNoErr;
	if((off_n != 0) && (off_m != 0))
	{
		app.width = off_n;
		app.height = off_m;

		value = sorg.ptr[0];
		ptr_temp_dst = dest->ptr;
		err5 = ippiSet_32f_C1R(value, ptr_temp_dst, dest->stp, app);

		value = sorg.ptr[sorg.n - 1];
		ptr_temp_dst += sorg.n + off_n;
		err6 = ippiSet_32f_C1R(value, ptr_temp_dst, dest->stp, app);

		value = sorg.ptr[(sorg.m - 1) * sorg.stp / sizeof(Ipp32f)];
		ptr_temp_dst = dest->ptr + ((off_m + sorg.m) * (dest->stp / temp));
		err7 = ippiSet_32f_C1R(value, ptr_temp_dst, dest->stp, app);

		value = sorg.ptr[((sorg.m - 1) * sorg.stp / sizeof(Ipp32f)) + sorg.n - 1];
		ptr_temp_dst += sorg.n + off_n;
		err8 = ippiSet_32f_C1R(value, ptr_temp_dst, dest->stp, app);
	}

	if ((err0 != ippStsOk) || (err1 != ippStsOk) || (err2 != ippStsOk) || (err3 != ippStsOk) || (err4 != ippStsOk) || (err5 != ippStsNoErr) || (err6 != ippStsNoErr) || (err7 != ippStsNoErr) || (err8 != ippStsNoErr))
		printf("ERRORE  repeat_padding");
}
void PopCod_Disp2D::convolution2D_valid_allocr(struct MatrixIpp matrix, struct MatrixIpp filter, struct MatrixIpp *result)
{
	IppStatus err;
	IppiSize m, f;

	result->m = matrix.m - filter.m + 1;
	result->n = matrix.n - filter.n + 1;
	m.width  = matrix.n;
	m.height = matrix.m;
	f.width  = filter.n;
	f.height = filter.m;

	result->ptr = ippiMalloc_32f_C1(result->n, result->m, &(result->stp));

	err = ippiConvValid_32f_C1R(matrix.ptr, matrix.stp, m, filter.ptr, filter.stp, f, result->ptr, result->stp);

	if (err != ippStsNoErr) 
		printf("ERRORE Convoluzione valid");
}


void PopCod_Disp2D::convolution2D_valid_allocc(struct MatrixIpp matrix, struct MatrixIpp filter, struct MatrixIpp *result)
{
	IppStatus err;
	IppiSize m, f;

	result->m = matrix.m - filter.n + 1;
	result->n = matrix.n - filter.m + 1;
	m.width  = matrix.n;
	m.height = matrix.m;
	f.width  = filter.m;
	f.height = filter.n;

	result->ptr = ippiMalloc_32f_C1(result->n, result->m, &(result->stp));

	err = ippiConvValid_32f_C1R(matrix.ptr, matrix.stp, m, filter.ptr, sizeof(Ipp32f*), f, result->ptr, result->stp);

	if (err != ippStsNoErr) 
		printf("ERRORE Convoluzione valid");
}
void PopCod_Disp2D::dimez_matrixipp_no_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest)
{
	IppStatus err;
	int sstr0,sstr1,sstr2,rstr0,rstr1,rstr2, temp;

	sstr0 = sorg.stp * sorg.m;
	sstr1 = sorg.stp * 2;
	sstr2 = sizeof(Ipp32f) * 2;
	rstr0 = dest->stp * dest->m;
	rstr1 = dest->stp;
	rstr2 = sizeof(Ipp32f);
	temp = 1;

	err = ippmCopy_ma_32f_SS(sorg.ptr, sstr0, sstr1, sstr2, dest->ptr, rstr0, rstr1, rstr2, dest->n, dest->m, temp);

	if (err != ippStsNoErr) 
		printf("ERRORE in dimez_mex_no_alloc");
}
//------------------------------------------------------------------------------------------------------------------------------------
//------spatial filtering--------------------------------------------------------------------------------------------------------------
void PopCod_Disp2D::filt_spatial_gabor_no_alloc(struct MatrixIpp imm, struct MatrixIpp *fil, struct MatrixIpp ****ris, const char *conv_type)
{
	struct MatrixIpp ***temp_ris;
	struct MatrixIpp imm_padd;
	struct MatrixIpp temp_r;  
	struct MatrixIpp temp_c;  
	struct MatrixIpp temp_1, temp_2, temp_3, temp_4;

	temp_ris = *ris;

	temp_1.m=temp_ris[0][0][0].m;
	temp_1.n=temp_ris[0][0][0].n;
	temp_1.ptr=ippiMalloc_32f_C1(temp_1.n,temp_1.m, &(temp_1.stp));

	temp_2.m=temp_ris[0][0][0].m;
	temp_2.n=temp_ris[0][0][0].n;
	temp_2.ptr=ippiMalloc_32f_C1(temp_2.n,temp_2.m, &(temp_2.stp));

	temp_3.m=temp_ris[0][0][0].m;
	temp_3.n=temp_ris[0][0][0].n;
	temp_3.ptr=ippiMalloc_32f_C1(temp_3.n,temp_3.m, &(temp_3.stp));

	temp_4.m=temp_ris[0][0][0].m;
	temp_4.n=temp_ris[0][0][0].n;
	temp_4.ptr=ippiMalloc_32f_C1(temp_4.n,temp_4.m, &(temp_4.stp));



	padding_alloc(imm, &imm_padd, fil[0].n, fil[0].n, conv_type);

	// 0 pi
	convolution2D_valid_allocc(imm_padd, fil[0], &temp_c);
	convolution2D_valid_no_allocr(temp_c, fil[1], &temp_ris[0][0][0]);
	convolution2D_valid_no_allocr(temp_c, fil[2], &temp_ris[1][0][0]);
	ippiFree(temp_c.ptr); 

	// 1/2 pi
	convolution2D_valid_allocr(imm_padd, fil[0], &temp_r);
	convolution2D_valid_no_allocc(temp_r, fil[1], &temp_ris[0][4][0]);
	convolution2D_valid_no_allocc(temp_r, fil[2], &temp_ris[1][4][0]);
	ippiFree(temp_r.ptr);    

	////////////////////////////////DIAGONALI///////////////////////////////////////////////////


	convolution2D_valid_allocc(imm_padd, fil[3], &temp_c);
	convolution2D_valid_no_allocr(temp_c, fil[3], &temp_1);
	convolution2D_valid_no_allocr(temp_c, fil[4], &temp_2);
	ippiFree(temp_c.ptr);
	convolution2D_valid_allocc(imm_padd, fil[4], &temp_c);
	convolution2D_valid_no_allocr(temp_c, fil[3], &temp_3);
	convolution2D_valid_no_allocr(temp_c, fil[4], &temp_4);
	ippiFree(temp_c.ptr);
	//1/4 pi
	subtraction_matrixipp_no_alloc(temp_1, temp_4, &temp_ris[0][2][0]);
	addition_matrixipp_no_alloc(temp_2, temp_3, &temp_ris[1][2][0]);

	// 3/4 pi
	addition_matrixipp_no_alloc(temp_1, temp_4, &temp_ris[0][6][0]);
	subtraction_matrixipp_no_alloc(temp_3, temp_2, &temp_ris[1][6][0]);

	////////////////////////////////CENTRALI////////////////////////////////////////////////

	convolution2D_valid_allocc(imm_padd, fil[7], &temp_c);
	convolution2D_valid_no_allocr(temp_c, fil[5], &temp_1);
	convolution2D_valid_no_allocr(temp_c, fil[6], &temp_2);
	ippiFree(temp_c.ptr);
	convolution2D_valid_allocc(imm_padd, fil[8], &temp_c);
	convolution2D_valid_no_allocr(temp_c, fil[5], &temp_3);
	convolution2D_valid_no_allocr(temp_c, fil[6], &temp_4);
	ippiFree(temp_c.ptr);
	// 1/8 pi
	subtraction_matrixipp_no_alloc(temp_1, temp_4, &temp_ris[0][1][0]);
	addition_matrixipp_no_alloc   (temp_2, temp_3, &temp_ris[1][1][0]);

	// 7/8 pi
	addition_matrixipp_no_alloc(temp_1, temp_4, &temp_ris[0][7][0]);
	subtraction_matrixipp_no_alloc(temp_3, temp_2, &temp_ris[1][7][0]);


	convolution2D_valid_allocc(imm_padd, fil[5], &temp_c);
	convolution2D_valid_no_allocr(temp_c, fil[7], &temp_1);
	convolution2D_valid_no_allocr(temp_c, fil[8], &temp_2);
	ippiFree(temp_c.ptr);
	convolution2D_valid_allocc(imm_padd, fil[6], &temp_c);
	convolution2D_valid_no_allocr(temp_c, fil[7], &temp_3);
	convolution2D_valid_no_allocr(temp_c, fil[8], &temp_4);
	ippiFree(temp_c.ptr);

	ippiFree(imm_padd.ptr);

	// 3/8 pi
	subtraction_matrixipp_no_alloc(temp_1, temp_4, &temp_ris[0][3][0]);
	addition_matrixipp_no_alloc(temp_2, temp_3, &temp_ris[1][3][0]);

	// 5/8 pi
	addition_matrixipp_no_alloc(temp_1, temp_4, &temp_ris[0][5][0]);
	subtraction_matrixipp_no_alloc(temp_3, temp_2, &temp_ris[1][5][0]);

	ippiFree(temp_1.ptr);
	ippiFree(temp_2.ptr);
	ippiFree(temp_3.ptr);    
	ippiFree(temp_4.ptr);
}
void PopCod_Disp2D::convolution2D_valid_no_allocr(struct MatrixIpp matrix, struct MatrixIpp filter, struct MatrixIpp *result)
{
	IppStatus err;
	IppiSize m, f;

	m.width  = matrix.n;
	m.height = matrix.m;
	f.width  = filter.n;
	f.height = filter.m;

	err = ippiConvValid_32f_C1R(matrix.ptr, matrix.stp, m, filter.ptr, filter.stp, f, result->ptr, result->stp);

	if (err != ippStsNoErr) 
		printf("ERRORE Convoluzione valid");
}


void PopCod_Disp2D::convolution2D_valid_no_allocc(struct MatrixIpp matrix, struct MatrixIpp filter, struct MatrixIpp *result)
{
	IppStatus err;
	IppiSize m, f;

	m.width  = matrix.n;
	m.height = matrix.m;
	f.width  = filter.m;
	f.height = filter.n;

	err = ippiConvValid_32f_C1R(matrix.ptr, matrix.stp, m, filter.ptr, sizeof(Ipp32f*), f, result->ptr, result->stp);

	if (err != ippStsNoErr) 
		printf("ERRORE Convoluzione valid");
}
void PopCod_Disp2D::addition_matrixipp_no_alloc(struct MatrixIpp a, struct MatrixIpp b, struct MatrixIpp *c)
{    
	IppStatus err;
	int s1str1, s1str2, s2str1, s2str2, rstr1, rstr2;
	int temp;

	temp = sizeof(Ipp32f);
	s1str1 = a.stp; 
	s1str2 = temp;
	s2str1 = b.stp;
	s2str2 = temp;
	rstr1 = c->stp;
	rstr2 = temp;

	err = ippmAdd_mm_32f(a.ptr, s1str1, s1str2, b.ptr, s2str1, s2str2, c->ptr, rstr1, rstr2, c->n, c->m);

	if (err != ippStsOk) 
		printf("ERRORE  in  addition_matrixipp_no_alloc");
}

void PopCod_Disp2D::subtraction_matrixipp_no_alloc(struct MatrixIpp a, struct MatrixIpp b, struct MatrixIpp *c)
{
	IppStatus err;
	int s1str1, s1str2, s2str1, s2str2, rstr1, rstr2;
	int temp;

	temp = sizeof(Ipp32f);
	s1str1 = a.stp; 
	s1str2 = temp;
	s2str1 = b.stp;
	s2str2 = temp;
	rstr1 = c->stp;
	rstr2 = temp;

	err = ippmSub_mm_32f(a.ptr, s1str1, s1str2, b.ptr, s2str1, s2str2, c->ptr, rstr1, rstr2, c->n, c->m);

	if (err != ippStsOk) 
		printf("ERRORE  in  subtraction_matrixipp_no_alloc");
}
//------------------------------------------------------------------------------------------------------------------------------------
//------shift_in_phase--------------------------------------------------------------------------------------------------------------
void PopCod_Disp2D::shift_in_phase(float ph_extr,float ph_step,struct MatrixIpp ***imm_filt ,  struct MatrixIpp ****ris, int n_orient)
{
	struct MatrixIpp ***temp, temp1,temp2;
	int h;
	int cont=0;
	float i, coseno, seno,j, ph_shift,m;

	temp1.m = imm_filt[0][0][0].m;
	temp1.n = imm_filt[0][0][0].n;
	temp1.ptr = ippiMalloc_32f_C1(temp1.n, temp1.m, &(temp1.stp));

	temp2.m = imm_filt[0][0][0].m;
	temp2.n = imm_filt[0][0][0].n;
	temp2.ptr = ippiMalloc_32f_C1(temp2.n, temp2.m, &(temp2.stp));


	temp = *ris;

	m=(float)(-2*PI_greco*fo);

	for(h=0 ; h<n_orient ; h++)
	{
		copy_matrixipp_no_alloc(imm_filt[0][h][0],&temp[0][h][3]);
		copy_matrixipp_no_alloc(imm_filt[1][h][0],&temp[1][h][3]);
	}

	for (i=(-ph_extr) ; i<=ph_extr ; i=j)
	{

		if(cont!=3)
		{
			for(h=0 ; h< n_orient; h++)
			{		
				ph_shift= m*i;
				coseno = (float)(cos(ph_shift));
				seno = (float)(sin(ph_shift));

				mul_with_constant_matrixipp_no_alloc(temp[0][h][3], &temp[0][h][cont], (Ipp32f)(coseno));

				mul_with_constant_matrixipp_no_alloc(temp[1][h][3], &temp1, (Ipp32f)(seno));

				mul_with_constant_matrixipp_no_alloc(temp[1][h][3], &temp[1][h][cont], (Ipp32f)(coseno)); 

				mul_with_constant_matrixipp_no_alloc(temp[0][h][3], &temp2, (Ipp32f)(seno));

				subtraction_matrixipp_inplace(&temp[0][h][cont], temp1);

				addition_matrixipp_inplace(&temp[1][h][cont], temp2);
			}

		}

		j=i+ph_step;
		cont++;

	}
	ippiFree(temp1.ptr);
	ippiFree(temp2.ptr);
}
void PopCod_Disp2D::mul_with_constant_matrixipp_no_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest, Ipp32f value)
{
	int temp, sstr1, sstr2, rstr1, rstr2;
	IppStatus err;    

	temp = sizeof(Ipp32f);
	sstr1 = sorg.stp;
	sstr2 = temp;
	rstr1 = dest->stp;
	rstr2 = temp;

	err = ippmMul_mc_32f(sorg.ptr, sstr1, sstr2, value, dest->ptr, rstr1, rstr2, dest->n, dest->m);

	if (err != ippStsOk) 
		printf("ERRORE  in  mul_with_costant_matrixipp_no_alloc");
}
void PopCod_Disp2D::subtraction_matrixipp_inplace(struct MatrixIpp *c, struct MatrixIpp a)
{
	IppiSize size;
	IppStatus err;

	size.width  = c->n;
	size.height = c->m;
	err = ippiSub_32f_C1IR(a.ptr, a.stp, c->ptr, c->stp, size);

	if (err != ippStsNoErr) 
		printf("ERRORE  in  subtraction_matrixipp_inplace");
}

void PopCod_Disp2D::addition_matrixipp_inplace(struct MatrixIpp *c, struct MatrixIpp a)
{
	IppiSize size;
	IppStatus err;

	size.width  = c->n;
	size.height = c->m;
	err = ippiAdd_32f_C1IR(a.ptr, a.stp, c->ptr, c->stp, size);

	if (err != ippStsNoErr) 
		printf("ERRORE  in  addition_matrixipp_inplace");
}
//------------------------------------------------------------------------------------------------------------------------------------
//------energy_no_alloc--------------------------------------------------------------------------------------------------------------
void PopCod_Disp2D::energy_no_alloc(struct MatrixIpp ****imm, struct MatrixIpp ***energy, struct MatrixIpp ****dec_energy, int n_orient, int n_phase, Ipp32f th)
{
	int h,k;
	struct MatrixIpp **temp_energy, ***dec_temp, temp_energy1, temp, temp1, dec_temp1;
	Ipp32f max, max_temp;
	Ipp32f max_temp1, max1;

	temp_energy = *energy;
	dec_temp = *dec_energy;


	dec_temp1.m = dec_energy[0][0][0][0].m;
	dec_temp1.n = dec_energy[0][0][0][0].n;
	dec_temp1.ptr = ippiMalloc_32f_C1(dec_temp1.n, dec_temp1.m, &(dec_temp1.stp));

	temp_energy1.m = energy[0][0][0].m;
	temp_energy1.n = energy[0][0][0].n;
	temp_energy1.ptr = ippiMalloc_32f_C1(temp_energy1.n, temp_energy1.m, &(temp_energy1.stp));

	temp1.m = energy[0][0][0].m;
	temp1.n = energy[0][0][0].n;
	temp1.ptr = ippiMalloc_32f_C1(temp1.n, temp1.m, &(temp1.stp));

	temp.m = energy[0][0][0].m;
	temp.n = energy[0][0][0].n;
	temp.ptr = ippiMalloc_32f_C1(temp.n, temp.m, &(temp.stp));

	for(h=0 ; h<n_orient ; h++)
	{
		//creiamo El

		sqr_matrixipp_no_alloc(imm[1][0][h][0],&dec_temp[h][1][0]);

		sqr_matrixipp_no_alloc(imm[1][1][h][0],&dec_temp1);

		addition_matrixipp_inplace(&dec_temp[h][1][0], dec_temp1);

		for(k=0 ; k< n_phase ; k++)
		{
			addition_matrixipp_no_alloc(imm[1][0][h][0],imm[0][0][h][k],&temp_energy1);
			sqr_matrixipp_no_alloc(temp_energy1,&temp_energy[h][k]);  
			addition_matrixipp_no_alloc(imm[1][1][h][0],imm[0][1][h][k],&temp1);
			sqr_matrixipp_no_alloc(temp1,&temp);
			addition_matrixipp_inplace(&temp_energy[h][k], temp);
			sqrt_matrixipp_inplace(&temp_energy[h][k]);

			//creiamo Er

			sqr_matrixipp_no_alloc(imm[0][0][h][k],&dec_temp[h][0][k]);
			sqr_matrixipp_no_alloc(imm[0][1][h][k],&dec_temp1);
			addition_matrixipp_inplace(&dec_temp[h][0][k], dec_temp1);
		}
	}

	max = dec_temp[0][1][0].ptr[0];
	max_temp1 = 0;
	max_temp=0;
	max1=dec_temp[0][0][0].ptr[0];
	for(h=0 ; h < n_orient ; h++)
	{
		max_of_matrixipp(dec_temp[h][1][0],&max_temp);

		if(max < max_temp)
			max = max_temp;
		for(k=0 ; k< n_phase ; k++)
		{
			max_of_matrixipp(dec_temp[h][0][k],&max_temp1);
			if(max1 < max_temp1)
				max1 = max_temp1;
		}
	}

	for(h=0 ; h<n_orient ; h++)
	{
		div_with_costant_matrixipp_inplace(&dec_temp[h][1][0],max);   // Er contiene ora l'energia normalizzata
		for(k=0 ; k< n_phase ; k++)
			div_with_costant_matrixipp_inplace(&dec_temp[h][0][k],max1);   // El contiene ora l'energia normalizzata
		//printf("--- h=%d k=%d\n",h,k);	
	}


	////////////////////////////////////////////creazione della maschera////////////////////////////////////////////////
	for(h=0 ; h<n_orient ; h++)
	{

		threshold_LTVal_inplace(&dec_temp[h][1][0], th, 0);

		threshold_GTVal_inplace(&dec_temp[h][1][0], th, 1);

		for(k=0 ; k< n_phase ; k++)
		{

			threshold_LTVal_inplace(&dec_temp[h][0][k], th, 0);

			threshold_GTVal_inplace(&dec_temp[h][0][k], th, 1);

		}
	}
	for(h=0 ; h<n_orient ; h++)
		for(k=0 ; k< n_phase ; k++)
		{


			and_matrixipp_inplace(&dec_temp[h][0][k], dec_temp[h][1][0]);

			mul_matrixipp_inplace(&temp_energy[h][k], dec_temp[h][0][k]);

		}

		/////////////////////////normalizzazione dell'energia////////////////////////////////////////7

		max_temp1 = 0;
		max1=0;//mdf
		for(h=0 ; h<n_orient ; h++)
			for(k=0 ; k< n_phase ; k++)
			{
				max_of_matrixipp(temp_energy[h][k],&max_temp1);
				if(max1 < max_temp1)
					max1 = max_temp1;
			}

			for(h=0 ; h<n_orient ; h++)
				for(k=0 ; k< n_phase ; k++)
					div_with_costant_matrixipp_inplace(&temp_energy[h][k],max1);


			ippiFree(temp.ptr);
			ippiFree(temp1.ptr);
			ippiFree(temp_energy1.ptr);
			ippiFree(dec_temp1.ptr);

}
void PopCod_Disp2D::sqr_matrixipp_no_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest)
{
	IppiSize size;
	IppStatus err;

	size.width  = sorg.n;
	size.height = sorg.m;
	err = ippiSqr_32f_C1R(sorg.ptr, sorg.stp, dest->ptr, dest->stp, size);

	if (err != ippStsNoErr) 
		printf("ERRORE  in  sqr_matrixipp_no_alloc ");
}
void PopCod_Disp2D::max_of_matrixipp(struct MatrixIpp matrix, Ipp32f *valmax)
{
	int i, j, temp;

	*valmax = 0;

	temp = matrix.stp / sizeof(Ipp32f);
	for(i = 0; i < matrix.m ; i++)
		for(j = 0 ; j < matrix.n ; j++)
			if((matrix.ptr[(i * temp) + j] > *valmax)) // && (!(mxIsNaN(matrix.ptr[(i * temp) + j]))))
				*valmax = matrix.ptr[(i * temp) + j];
}
void PopCod_Disp2D::div_with_costant_matrixipp_inplace(struct MatrixIpp *matrix, Ipp32f value)
{
	IppiSize size;
	IppStatus err;

	size.width  = matrix->n;
	size.height = matrix->m;
	err = ippiDivC_32f_C1IR(value, matrix->ptr, matrix->stp, size);

	if (err != ippStsNoErr) 
		printf("ERRORE  in  div_with_costant_matrixipp_inplace ");
}
void PopCod_Disp2D::threshold_LTVal_inplace(struct MatrixIpp *matrix, Ipp32f threshold, Ipp32f value)  
{
	IppiSize size;
	IppStatus err;

	size.width  = matrix->n;
	size.height = matrix->m;

	err = ippiThreshold_LTVal_32f_C1IR(matrix->ptr, matrix->stp, size, threshold, value);

	if (err != ippStsNoErr) 
		printf("ERRORE  in  threshold_LTVal_tozero ");
}

void PopCod_Disp2D::threshold_GTVal_inplace(struct MatrixIpp *matrix, Ipp32f threshold, Ipp32f value)  
{
	IppiSize size;
	IppStatus err;

	size.width  = matrix->n;
	size.height = matrix->m;

	err = ippiThreshold_GTVal_32f_C1IR(matrix->ptr, matrix->stp, size, threshold, value);

	if (err != ippStsNoErr) 
		printf("ERRORE  in  threshold_GTVal_toone ");
}
void PopCod_Disp2D::and_matrixipp_inplace(struct MatrixIpp *c, struct MatrixIpp a)  
{
	//mdf
	int temp = c->stp / sizeof(Ipp32f);
    for(int i = 0; i < c->m ; i++)
        for(int j = 0 ; j < c->n ; j++)
			if(c->ptr[(i * temp) + j]==1 && a.ptr[(i * temp) + j]==1)
				c->ptr[(i * temp) + j]=1;
			else 
				c->ptr[(i * temp) + j]=0;
}
void PopCod_Disp2D::mul_matrixipp_inplace(struct MatrixIpp *a, struct MatrixIpp b)
{
	IppiSize size;
	IppStatus err;

	size.width  = b.n;
	size.height = b.m;

	err = ippiMul_32f_C1IR(b.ptr, b.stp, a->ptr, a->stp, size);
	if (err != ippStsNoErr) 
		printf("ERRORE  in  mul_matrixipp_inplace ");
}
void PopCod_Disp2D::sqrt_matrixipp_inplace(struct MatrixIpp *matrix)
{
	IppiSize size;
	IppStatus err;

	size.width  = matrix->n;
	size.height = matrix->m;
	err = ippiSqrt_32f_C1IR(matrix->ptr, matrix->stp, size);

	if (err != ippStsNoErr) 
		printf("ERRORE  in  sqrt_matrixipp_inplace ");
}
//--------------------------------------------------------------------------------------------------------------------------
//------warping--------------------------------------------------------------------------------------------------------------
void PopCod_Disp2D::warping_frame_filt_gabor(struct MatrixIpp ****buf, struct MatrixIpp *****buf_copy, struct MatrixIpp *full_disp, int n_frame, int n_orient,int n_phase)
{
	struct MatrixIpp ****copy;
	struct MatrixIpp temp_dispX, temp_dispY;
	int j, w,k;

	copy= *buf_copy;



	temp_dispX=full_disp[Vx];
	temp_dispY=full_disp[Vy];

	for(j = 0 ; j < 2 ; j++)
		for(w = 0 ; w < n_orient ; w++)
		{
			for(k=0; k<n_phase; k++)
				warping(buf[DX][j][w][k], temp_dispX, temp_dispY, &copy[DX][j][w][k]); 
			//copy_matrixipp_no_alloc(buf[DX][j][w][k], &copy[DX][j][w][k]);//MDF

			copy_matrixipp_no_alloc(buf[SX][j][w][0], &copy[SX][j][w][0]);
		}
}
void PopCod_Disp2D::warping(struct MatrixIpp sorg, struct MatrixIpp dx, struct MatrixIpp dy, struct MatrixIpp *dest)
{
	int i, j;
	int x_w, y_w;
	Ipp32f off_x, off_y;
	int dummy, off, off_w, app;

	app = dest->stp / sizeof(ipp32f);
	for (i = 0 ; i < dest->m ; i++)
	{ 
		dummy = i * app;
		for (j = 0 ; j < dest->n ; j++)
		{ 
			off = j + dummy;
			dest->ptr[off] = 0;
			y_w = i - ((int)(dy.ptr[off]));//floor((int)(dy.ptr[off]));
			x_w = j - ((int)(dx.ptr[off]));




			if((y_w < dest->m - 1) && (y_w >= 0) && (x_w < dest->n - 1) && (x_w >= 0))
			{

				off_w = x_w + (y_w * app); 
				off_x = - (Ipp32f)(x_w) - dx.ptr[off] + j; //xs
				off_y = - (Ipp32f)(y_w) - dy.ptr[off] + i; //ys


				//interpolazione bilineare
				dest->ptr[off] =  sorg.ptr[off_w]           * ((1 - off_x) * (1 - off_y));
				dest->ptr[off] += sorg.ptr[off_w + 1]       * ((off_x)     * (1 - off_y));
				dest->ptr[off] += sorg.ptr[off_w + app]     * ((1 - off_x) * (off_y));
				dest->ptr[off] += sorg.ptr[off_w + app + 1] * ((off_x)     * (off_y)); 
			}
		}
	}
}
//------------------------------------------------------------------------------------------------------------------------------------
//---------population--------------------------------------------------------------------------------------------------------------------
void PopCod_Disp2D::population_no_alloc(struct MatrixIpp **energy, struct MatrixIpp disparity, struct MatrixIpp **population, struct MatrixIpp **mask, int n_orient, int n_phase)
{
	struct MatrixIpp *temp_population, *temp_mask;
	int  r, c,k, h, temp; 
	Ipp32f  max_temp1,min_temp1,num, den;

	temp_population = *population;
	temp_mask=*mask;

	temp=energy[0][0].stp/sizeof(Ipp32f);

	for(h=0 ; h<n_orient ; h++)
		for(r=0 ; r<energy[0][0].m ; r++)
			for(c=0 ; c<energy[0][0].n ; c++)
			{
				max_temp1 = energy[h][0].ptr[(r*temp) +c];
				min_temp1 = energy[h][0].ptr[(r*temp) +c];
				num=0;
				den=0;

				for(k=1 ; k< n_phase ; k++)
				{
					if(max_temp1 < energy[h][k].ptr[(r*temp) +c])
						max_temp1=energy[h][k].ptr[(r*temp) +c];
					if(min_temp1 > energy[h][k].ptr[(r*temp) +c])
						min_temp1=energy[h][k].ptr[(r*temp) +c];
				}

				if(max_temp1!=0)
				{			
					for(k=0 ; k< n_phase ; k++)
					{
						energy[h][k].ptr[(r * temp) + c]=((energy[h][k].ptr[(r*temp) +c])-min_temp1)/(max_temp1-min_temp1);

						if(energy[h][k].ptr[(r * temp) + c]<0.7)									
							energy[h][k].ptr[(r * temp) + c]=0;

						num += disparity.ptr[k] * (energy[h][k].ptr[(r * temp) + c]);				//calcoliamo numeratore di d per ogni pixel
						den += energy[h][k].ptr[(r * temp) + c];							//calcoliamo denominatore di d per ogni pixel
					}

					temp_population[h].ptr[(r * temp) + c] = num/ den;	//calcoliamo d
					temp_mask[h].ptr[(r * temp) + c]=1;					
				}
				else{
					temp_mask[h].ptr[(r * temp) + c]=0;	
					temp_population[h].ptr[(r * temp) + c] = 0;//mdf
				}
			}

}
void PopCod_Disp2D::decomposition_of_population_no_alloc(struct MatrixIpp *population, struct MatrixIpp ***decompose_population, int n_orient)
{
	struct MatrixIpp **temp_population;
	int i; 
	Ipp32f cos_alfa, sin_alfa, alfa, temp_alfa;

	temp_population = *decompose_population;

	alfa = (PI_greco ) / n_orient;

	for(i = 0 ; i < n_orient ; i++)
	{
		temp_alfa = i * alfa;
		cos_alfa = cos(temp_alfa);
		sin_alfa = sin(temp_alfa);
		mul_with_constant_matrixipp_no_alloc(population[i], &temp_population[i][Vx], cos_alfa);
		mul_with_constant_matrixipp_no_alloc(population[i], &temp_population[i][Vy], sin_alfa);
	}
}
//------------------------------------------------------------------------------------------------------------------------------------
//---------fulldisparity--------------------------------------------------------------------------------------------------------------------
void PopCod_Disp2D::fulldisparity_no_alloc(struct MatrixIpp **dec_population, struct MatrixIpp **fulldisparity, struct MatrixIpp *mask,struct MatrixIpp *mask2, int n_orient, int nc_min)
{
	struct MatrixIpp *temp_fulldisp, temp_mask2;
	Ipp32f *vvx, *vvy, *L_2;
	Ipp32f SumX, SumY, SumXYL_2, SumXXL_2, SumYYL_2, den;
	int i, j, k, temp, count;
	IppiSize size; 
	IppStatus err1, err2;
	float EPS_float=1e-6f;

	temp_fulldisp = *fulldisparity;

	temp_mask2.m = temp_fulldisp[0].m;
	temp_mask2.n = temp_fulldisp[0].n;
	temp_mask2.ptr = ippiMalloc_32f_C1(temp_mask2.n, temp_mask2.m, &(temp_mask2.stp));

	mul_with_constant_matrixipp_no_alloc(temp_mask2,&temp_mask2,0);

	size.width =  temp_fulldisp[Vx].n;
	size.height = temp_fulldisp[Vx].m;
	err1 = ippiSet_32f_C1R(0, temp_fulldisp[Vx].ptr, temp_fulldisp[Vx].stp, size);
	err2 = ippiSet_32f_C1R(0, temp_fulldisp[Vy].ptr, temp_fulldisp[Vy].stp, size);
	if ((err1 != ippStsOk) || (err2 != ippStsOk))
		printf("ERRORE  fulldisparity_no_alloc");

	vvx = (Ipp32f *)malloc(n_orient * sizeof(Ipp32f)); 
	vvy = (Ipp32f *)malloc(n_orient * sizeof(Ipp32f));
	L_2 = (Ipp32f *)malloc(n_orient * sizeof(Ipp32f));

	temp = temp_fulldisp[0].stp / sizeof(Ipp32f);

	for(i = 0 ; i < temp_fulldisp[Vx].m ; i++)
		for(j = 0 ; j < temp_fulldisp[Vx].n ; j++)
		{
			count = n_orient;
			for(k = 0 ; k < n_orient ; k++)
			{
				vvx[k] = dec_population[k][Vx].ptr[(i * temp) + j];       
				vvy[k] = dec_population[k][Vy].ptr[(i * temp) + j];
				L_2[k] = (vvx[k] * vvx[k]) + (vvy[k] * vvy[k]);
				if((L_2[k] < EPS_float)||(mask[k].ptr[(i * temp) + j]==0))
				{
					count--;
					vvx[k] = 0;
					vvy[k] = 0;
					L_2[k] = 1;
				}
			}

			if(count >= nc_min)
			{   
				SumX = 0;
				SumY = 0;
				SumXYL_2 = 0;
				SumXXL_2 = 0;
				SumYYL_2 = 0;
				for(k = 0 ; k < n_orient ; k++)
				{
					SumX += vvx[k];
					SumY += vvy[k];
					SumXYL_2 += (vvx[k] * vvy[k]) / L_2[k];
					SumXXL_2 += (vvx[k] * vvx[k]) / L_2[k];
					SumYYL_2 += (vvy[k] * vvy[k]) / L_2[k];
				}
				den = (SumXYL_2 * SumXYL_2) - (SumXXL_2 * SumYYL_2);

				if (den != 0)
				{
					temp_fulldisp[Vx].ptr[(i * temp) + j] = -((SumX * SumYYL_2) - (SumY * SumXYL_2)) / den;
					temp_fulldisp[Vy].ptr[(i * temp) + j] =  ((SumX * SumXYL_2) - (SumY * SumXXL_2)) / den;
					temp_mask2.ptr[(i * temp) + j]=1;
				}
			}

		}

		if (mask2->ptr!=0){ ippiFree(mask2->ptr); mask2->ptr=0;}//mdf
		*mask2=temp_mask2;

		free(vvx);
		free(vvy);
		free(L_2);

}
void PopCod_Disp2D::expand_mulX2_fulldisparity_no_alloc(struct MatrixIpp *full_in, struct MatrixIpp **full_out)
{
	struct MatrixIpp *temp_full;
	int i;

	temp_full = *full_out;

	for(i = 0 ; i < 2 ; i++)
	{
		expand_no_alloc(full_in[i], &temp_full[i]);
		mul_with_costant_matrixipp_inplace(&temp_full[i], 2);
	}
}
void PopCod_Disp2D::mul_with_costant_matrixipp_inplace(struct MatrixIpp *matrix, Ipp32f value)
{
	IppiSize size;
	IppStatus err;

	size.width  = matrix->n;
	size.height = matrix->m;
	err = ippiMulC_32f_C1IR(value, matrix->ptr, matrix->stp, size);

	if (err != ippStsNoErr) 
		printf("ERRORE  in  mul_with_costant_matrixipp_inplace ");
}
void PopCod_Disp2D::expand_mask(struct MatrixIpp mask, struct MatrixIpp *mask_out, int m, int n)
{
	int i,j,temp;
	struct MatrixIpp temp_mask_out;

	temp_mask_out.m=m;
	temp_mask_out.n=n;
	temp_mask_out.ptr = ippiMalloc_32f_C1(temp_mask_out.n, temp_mask_out.m, &(temp_mask_out.stp));

	temp=temp_mask_out.stp/sizeof(Ipp32f);

	expand_no_alloc(mask, &temp_mask_out);
	for(i=0;i<temp_mask_out.m;i++)
		for(j=0;j<temp_mask_out.n;j++)
		{
			if(temp_mask_out.ptr[(i*temp) + j] >= 0.5)
				temp_mask_out.ptr[(i*temp) + j]=1;
			else
				temp_mask_out.ptr[(i*temp) + j]=0;
		}
		if (mask_out->ptr!=0) {ippiFree(mask_out->ptr); mask_out->ptr=0;}//mdf

		*mask_out=temp_mask_out;



}
void PopCod_Disp2D::expand_no_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest)
{
	int j,k;
	int num, app_1, app_2, app_3;
	int sstr0, sstr1, sstr2, rstr0, rstr1, rstr2;
	IppStatus err;

	num = 1;   
	sstr0 = sorg.m * sorg.stp;
	sstr1 = sorg.stp;
	sstr2 = sizeof(Ipp32f);
	rstr0 = dest->m * dest->stp;
	rstr1 = 2 * dest->stp;
	rstr2 = 2 * sizeof(Ipp32f);

	err = ippmCopy_ma_32f_SS(sorg.ptr, sstr0, sstr1, sstr2, dest->ptr, rstr0, rstr1, rstr2, sorg.n, sorg.m, num);

	if (err != ippStsOk) 
		printf("ERRORE expand_no_alloc");

	app_1 = dest->stp / sizeof(ipp32f);
	for (k = 0 ; k < dest->m - 2 ; k += 2)
	{
		app_2 = (k * app_1);

		for (j = 0 ; j < dest->n - 2 ; j += 2)
		{
			app_3 = j + app_2;
			dest->ptr[app_3 + 1]         = ( dest->ptr[app_3] + dest->ptr[app_3 + 2]) / 2;
			dest->ptr[app_3 + app_1]     = ( dest->ptr[app_3] + dest->ptr[app_3 + (2 * app_1)] ) / 2;
			dest->ptr[app_3 + app_1 + 1] = ( dest->ptr[app_3] + dest->ptr[app_3 + 2] + dest->ptr[app_3 + (2 * app_1)] + dest->ptr[app_3 + (2 * app_1) + 2]) / 4;
		}

		app_3 = j + app_2;
		dest->ptr[app_3 + 1]         = dest->ptr[app_3];
		dest->ptr[app_3 + app_1]     = ( dest->ptr[app_3] + dest->ptr[app_3 + (2 * app_1)] ) / 2;
		dest->ptr[app_3 + app_1 + 1] = dest->ptr[app_3 + app_1];
	}

	app_2 = (k * app_1);
	for (j = 0 ; j < dest->n - 2 ; j += 2)
	{
		app_3 = j + app_2;
		dest->ptr[app_3 + 1]         = ( dest->ptr[app_3] + dest->ptr[app_3 + 2]) / 2;
		dest->ptr[app_3 + app_1]     = dest->ptr[app_3];
		dest->ptr[app_3 + app_1 + 1] = dest->ptr[app_3 + 1];
	}
	app_3 = j + app_2;
	dest->ptr[app_3 + 1]         = dest->ptr[app_3];
	dest->ptr[app_3 + app_1]     = dest->ptr[app_3];
	dest->ptr[app_3 + app_1 + 1] = dest->ptr[app_3];
}
void PopCod_Disp2D::merge(struct MatrixIpp *full_disp ,struct MatrixIpp **fullmerged)
{
	struct MatrixIpp *merged;
	int   k, temp;

	merged = *fullmerged;

	temp = full_disp[0].stp / sizeof(Ipp32f);

	for(k = 0 ; k < 2 ; k++)
		addition_matrixipp_inplace(&merged[k], full_disp[k]);
}
void PopCod_Disp2D::cut_invalid_at_end_flow(struct MatrixIpp mask ,struct MatrixIpp **fullmerged,int nc_min)
{
	struct MatrixIpp *merged;
	int i, j, temp;

	merged = *fullmerged;

	temp = merged[0].stp / sizeof(Ipp32f);

	for(i = 0 ; i < merged[0].m ; i++)
		for(j = 0 ; j < merged[0].n ; j++)
			if(mask.ptr[(i*temp)+j]==0)
			{
				merged[Vx].ptr[(i * temp) + j] = undi;//unreliable disparity
				merged[Vy].ptr[(i * temp) + j] = undi;//unreliable disparity
			}

}